@@ -11,6 +11,13 @@ use Clone;
 use List::Util qw(min max first) ;
 use List::MoreUtils qw(any minmax first_value) ;
 
+use Glib ':constants';
+use Gtk2 -init;
+use Gtk2::Gdk::Keysyms ;
+my %K = %Gtk2::Gdk::Keysyms ;
+my %C = map{$K{$_} => $_} keys %K ;
+
+
 use App::Asciio::Setup ;
 use App::Asciio::Dialogs ;
 use App::Asciio::Elements ;
@@ -23,7 +30,7 @@ use App::Asciio::Options ;
 
 #-----------------------------------------------------------------------------
 
-our $VERSION = '1.51' ;
+our $VERSION = '1.01' ;
 
 #-----------------------------------------------------------------------------
 
@@ -60,15 +67,18 @@ App::Asciio - Plain ASCII diagram
 
 =head1 DESCRIPTION
 
-It has always been painful to do ASCII diagrams by hand. This perl application allows 
-you to draw ASCII diagrams in a modern (but simple) graphical interface.
+This gtk2-perl application allows you to draw ASCII diagrams in a modern (but simple) graphical
+application. The ASCII graphs can be saved as ASCII or in a format that allows you to modify them later.
 
-The ASCII graphs can be saved as ASCII or in a format that allows you to modify them later.
+Thanks to all the Perl-QA hackathon 2008 in Oslo for pushing me to do an early release.
 
 Special thanks go to the Muppet and the gtk-perl group, Gbor Szab for his help and advices.
 
 Adam Kennedy coined the cool name.
 
+Sometimes a diagram is worth a lot of text in a source code file. It has always been painfull to do 
+ASCII diagrams by hand. 
+
 =head1 DOCUMENTATION
 
 =head2 Asciio user interface
@@ -83,8 +93,8 @@ Adam Kennedy coined the cool name.
             | .............| Rulers    > || computer > || text         |..... |
             | .............| File      > || people   > || wirl_arrow   |..... |
      grid---------->.......'-------------'| divers   > || axis         |..... |
-            | ......................^.....'------------'| boxes      > |..... |
-            | ......................|...................| rulers     > |..... |
+            | ............................'------------'| boxes      > |..... |
+            | ......................^...................| rulers     > |..... |
             | ......................|...................'--------------'..... |
             | ......................|........................................ |
             | ......................|........................................ |
@@ -96,11 +106,9 @@ Adam Kennedy coined the cool name.
                               context menu
 				   
 
-Press 'F1' for help.
-
 =head2 context menu
 
-The context menu allows to access to B<Asciio> commands.
+The context menu allows to access to B<asciio> commands. B<ASCII> is used to insert ASCII elements.
 
 =head2 keyboard shortcuts
 
@@ -121,19 +129,13 @@ implemented are:
 
 =item * local clipboard operations
 
-=item * send to front/back
-
-=item * insert arrow, boxes, text
-
-=item * ...
-
 =back
 
-The available commands are displayed if you press B<K>. 
+A window displaying the currently available commands is displayed if you press B<K>.
 
 =head2 elements
 
-There are a few elements implemented at the moment.
+There but a few elements implemented at the moment.
 
 =head3 wirl arrow
 
@@ -152,50 +154,11 @@ An arrow that tries to do what you want. Try rotating the end clockwise then cou
                  /
                 /
                v
-	       
-
-=head3 multi section wirl arrow
-
-A set of whirl arrows connected to each other
-
- .----------.                       .
- |          |                \     / \
-    .-------'           ^     \   /   \
-    |                    \     \ /     \
-    |   .----------->     \     '       .
-    |   '----.             \           /
-    |        |              \         /
-    '--------'               '-------'
-    
-
-
-=head3 angled arrow and axis
-
-   -------.         .-------
-           \       /
-            \     /
-             \   /
- 
-             /   \
-            /     \
-           /       \
-    ------'         '-------
-    
-          ^
-     ^    |    ^
-      \   |   /
-       \  |  /
-        \ | /
- <-------- -------->
-        / |\
-       /  | \
-      /   |  \
-     v    |   v
-          v
-	
 
 =head3 box and text 
 
+Both are implemented within the same code. Try double clicking on a box to see what you can do with it.
+
                  .----------.
                  |  title   |
   .----------.   |----------|   ************
@@ -206,28 +169,15 @@ A set of whirl arrows connected to each other
                                  (\_/)               |
          edit_me                 (O.o)  <------------'
                                  (> <)
-				 
-
-You can also use the 'External commands in box' to direct an external command output to a box. Default shortcuts are 'x' and CTL + 'x'.
-
-=head3 "if" box and "process" box
-
-                        ____________
-   .--------------.     \           \
-  / a == b         \     \           \   __________
- (    &&            )     ) process   )  \         \
-  \ 'string' ne '' /     /           /    ) process )
-   '--------------'     /___________/    /_________/
-   
 
 =head3 your own stencils
 
-Take a look at I<setup/stencils/computer> for a stencil example. Stencils listed in I<setup/setup.ini> will
+Take a look at I<setup/stencils/computer> for a stencil example. Stencils lites in I<setup/setup.ini> will
 be loaded when B<Asciio> starts.
 
 =head3 your own element type
 
-For simple elements, put your design in a box. That should cover 90% of anyone's needs. You can look in 
+For simple elemnts, put your design in a box. that should cover 90% of anyone's needs. You can look in 
 I<lib/stripes> for element implementation examples.
 
 =head2 exporting to ASCII
@@ -249,6 +199,8 @@ Exporting to the clipboard is done with B<ctl + e>.
                       /          \
           User code  v            v OS code
 	  
+
+
 	
 	     .---.  .---. .---.  .---.    .---.  .---.
     OS API   '---'  '---' '---'  '---'    '---'  '---'
@@ -294,61 +246,18 @@ Exporting to the clipboard is done with B<ctl + e>.
                                '---------'
 			       
 
-                                        .--Base::Class::Derived_A
-                                       /
-                                      .----Base::Class::Derived_B    
-      Something--------.             /         \
-                        \           /           '---Base::Class::Derived::More
-      Something::else    \         /             \
-            \             \       /               '-Base::Class::Derived::Deeper
-             \             \     /
-              \             \   .-----------Base::Class::Derived_C 
-               \             \ /
-                '-------Base::Class
-                       /   \ \ \
-                      '     \ \ \
-                      |      \ \ '---The::Latest
-                     /|       \ \      \
- With::Some::fantasy' '        \ \      '----The::Latest::Greatest
-                     /|         \ \
-         More::Stuff' '          \ '-I::Am::Running::Out::Of::Ideas
-                     /|           \
-         More::Stuff' '            \
-                     /              '---Last::One
-         More::Stuff'
-
-
-   ____[]
-  | ___ |
-  ||   ||  device
-  ||___||  loads
-  | ooo |------------------------------------------------------------.
-  | ooo |    |                          |                            |
-  | ooo |    |                          |                            |
-  '_____'    |                          |                            |
-             |                          |                            |
-             v                          v                            v
-   .-------------------.  .---------------------------.    .-------------------.
-   | Loadable module C |  |     Loadable module A     |    | Loadable module B |
-   '-------------------'  |---------------------------|    |   (instrumented)  |
-             |            |         .-----.           |    '-------------------'
-             '--------------------->| A.o |           |              |
-                 calls    |         '-----'           |              |
-                          |    .------------------.   |              |
-                          |    | A.instrumented.o |<-----------------'
-                          |    '------------------'   |    calls
-                          '---------------------------'
-
-
 =cut
 
 sub new
 {
-my ($class) = @_ ;
+my ($class, $width, $height) = @_ ;
+
+my $drawing_area = Gtk2::DrawingArea->new;
 
 my $self = 
 	bless 
 		{
+		widget => $drawing_area,
 		ELEMENT_TYPES => [],
 		ELEMENTS => [],
 		CONNECTIONS => [],
@@ -364,7 +273,6 @@ my $self =
 		DRAGGING => '',
 		SELECTION_RECTANGLE =>{START_X => 0, START_Y => 0},
 		
-		KEYS => {K =>{}, C =>{},},
 		ACTIONS => {},
 		VALID_SELECT_ACTION => { map {$_, 1} qw(resize move)},
 		
@@ -393,7 +301,6 @@ my $self =
 			connection => 'Chocolate',
 			connection_point => [230, 198, 133],
 			connector_point => 'DodgerBlue',
-			new_connection => 'red' ,
 			extra_point => [230, 198, 133],
 			},
 		
@@ -405,7 +312,29 @@ my $self =
 		
 		DO_STACK_POINTER => 0,
 		DO_STACK => [] ,
-		}, $class ;
+		}, __PACKAGE__ ;
+
+$drawing_area->can_focus(TRUE) ;
+
+$drawing_area->signal_connect(configure_event => \&configure_event, $self);
+$drawing_area->signal_connect(expose_event => \&expose_event, $self);
+$drawing_area->signal_connect(motion_notify_event => \&motion_notify_event, $self);
+$drawing_area->signal_connect(button_press_event => \&button_press_event, $self);
+$drawing_area->signal_connect(button_release_event => \&button_release_event, $self);
+$drawing_area->signal_connect(key_press_event => \&key_press_event, $self);
+
+$drawing_area->set_events
+		([qw/
+		exposure-mask
+		leave-notify-mask
+		button-press-mask
+		button-release-mask
+		pointer-motion-mask
+		key-press-mask
+		key-release-mask
+		/]);
+
+$self->event_options_changed() ;
 
 return($self) ;
 }
@@ -426,12 +355,22 @@ $self->set_font($self->{FONT_FAMILY}, $self->{FONT_SIZE});
 
 #-----------------------------------------------------------------------------
 
+sub destroy
+{
+my ($self) = @_;
+
+$self->{widget}->get_toplevel()->destroy() ;
+}
+
+#-----------------------------------------------------------------------------
+
 sub set_title
 {
 my ($self, $title) = @_;
 
 if(defined $title)
 	{
+	$self->{widget}->get_toplevel()->set_title($title . ' - asciio') ;
 	$self->{TITLE} = $title ;
 	}
 }
@@ -450,6 +389,14 @@ my ($self, $font_family, $font_size) = @_;
 
 $self->{FONT_FAMILY} = $font_family || 'Monospace';
 $self->{FONT_SIZE} = $font_size || 10 ;
+
+$self->{widget}->modify_font
+	(
+	Gtk2::Pango::FontDescription->from_string 
+		(
+		$self->{FONT_FAMILY} . ' ' . $self->{FONT_SIZE}
+		)
+	);
 }
 
 sub get_font
@@ -464,43 +411,11 @@ return($self->{FONT_FAMILY},  $self->{FONT_SIZE}) ;
 sub update_display 
 {
 my ($self) = @_;
+my $widget = $self->{widget} ;
 
 $self->call_hook('CANONIZE_CONNECTIONS', $self->{CONNECTIONS}, $self->get_character_size()) ;
-}
 
-sub get_grid_usage
-{
-my ($self) = @_;
-
-my %cost_map ;
-
-# todo: keep previous cost map and update only changed elements
-
-for my $element (@{$self->{ELEMENTS}})
-	{
-	for my $mask_and_element_strip ($element->get_mask_and_element_stripes())
-		{
-		my $x_offset = $element->{X} + $mask_and_element_strip->{X_OFFSET} ;
-		my $y_offset = $element->{Y} + $mask_and_element_strip->{Y_OFFSET} ;
-		
-		my $string_offset = 0 ;
-		
-		for my $string ( split /\n/, $mask_and_element_strip->{TEXT})
-			{
-			for (0 .. length($string) - 1 )
-				{
-				my $x_offset_line = $x_offset + $_ ;
-				my $y_offset_line = ($y_offset + $string_offset)  ;
-				
-				$cost_map{"$x_offset_line.$y_offset_line"} = 0 ;
-				}
-				
-			$string_offset++ ;
-			}
-		}
-	}
-
-return \%cost_map ;
+$widget->queue_draw_area(0, 0, $widget->allocation->width,$widget->allocation->height);
 }
 
 #-----------------------------------------------------------------------------
@@ -512,19 +427,352 @@ my ($self, $hook_name,  @arguments) = @_;
 $self->{HOOKS}{$hook_name}->(@arguments)  if (exists $self->{HOOKS}{$hook_name}) ;
 }
 
+#-----------------------------------------------------------------------------
+
+sub configure_event 
+{
+my ($widget, $event, $self) = @_;
+
+$self->{PIXMAP} = Gtk2::Gdk::Pixmap->new 
+			(
+			$widget->window,
+			$widget->allocation->width, $widget->allocation->height,
+			-1
+			);
+
+$self->{PIXMAP}->draw_rectangle
+		(
+		$widget->get_style->base_gc ($widget->state),
+		TRUE, 0, 0, $widget->allocation->width,
+		$widget->allocation->height
+		);
+		
+return TRUE;
+}
+
+#-----------------------------------------------------------------------------
+
+sub expose_event
+{
+my ($widget, $event, $self) = @_;
+
+my $gc = Gtk2::Gdk::GC->new($self->{PIXMAP});
+
+# draw background
+$gc->set_foreground($self->get_color('background'));
+
+$self->{PIXMAP}->draw_rectangle
+		(
+		$gc,	TRUE,
+		0, 0,
+		$widget->allocation->width, $widget->allocation->height
+		);
+
+my ($character_width, $character_height) = $self->get_character_size() ;
+my ($widget_width, $widget_height) = $self->{PIXMAP}->get_size();
+
+if($self->{DISPLAY_GRID})
+	{
+	$gc->set_foreground($self->get_color('grid'));
+
+	for my $horizontal (0 .. ($widget_height/$character_height) + 1)
+		{
+		$self->{PIXMAP}->draw_line
+					(
+					$gc,
+					0,  $horizontal * $character_height,
+					$widget_width, $horizontal * $character_height
+					);
+		}
+
+	for my $vertical(0 .. ($widget_width/$character_width) + 1)
+		{
+		$self->{PIXMAP}->draw_line
+					(
+					$gc,
+					$vertical * $character_width, 0,
+					$vertical * $character_width, $widget_height
+					);
+		}
+	}
+	
+# draw elements
+for my $element (@{$self->{ELEMENTS}})
+	{
+	my ($background_color, $foreground_color) =  $element->get_colors() ;
+	
+	if($self->is_element_selected($element))
+		{
+		if(exists $element->{GROUP} and defined $element->{GROUP}[-1])
+			{
+			$background_color = 
+				$self->get_color
+					(
+					$self->{COLORS}{group_colors}[$element->{GROUP}[-1]{GROUP_COLOR}][0]
+					) ;
+			}
+		else
+			{
+			$background_color = $self->get_color('selected_element_background');
+			}
+		}
+	else
+		{
+		if(defined $background_color)
+			{
+			$background_color = $self->get_color($background_color) ;
+			}
+		else
+			{
+			if(exists $element->{GROUP} and defined $element->{GROUP}[-1])
+				{
+				$background_color = 
+					$self->get_color
+						(
+						$self->{COLORS}{group_colors}[$element->{GROUP}[-1]{GROUP_COLOR}][1]
+						) ;
+				}
+			else
+				{
+				$background_color = $self->get_color('element_background') ;
+				}
+			}
+		}
+			
+	$foreground_color = 
+		defined $foreground_color
+			? $self->get_color($foreground_color)
+			: $self->get_color('element_foreground') ;
+			
+	$gc->set_foreground($foreground_color);
+	
+	for my $mask_and_element_strip ($element->get_mask_and_element_stripes())
+		{
+		$gc->set_foreground($background_color);
+			
+		$self->{PIXMAP}->draw_rectangle
+					(
+					$gc,
+					$self->{OPAQUE_ELEMENTS},
+					($element->{X} + $mask_and_element_strip->{X_OFFSET}) * $character_width,
+					($element->{Y} + $mask_and_element_strip->{Y_OFFSET}) * $character_height,
+					$mask_and_element_strip->{WIDTH} * $character_width,
+					$mask_and_element_strip->{HEIGHT} * $character_height,
+					);
+					
+		$gc->set_foreground($foreground_color);
+		
+		my $layout = $widget->create_pango_layout($mask_and_element_strip->{TEXT}) ;
+		
+		my ($text_width, $text_height) = $layout->get_pixel_size;
+		
+		$self->{PIXMAP}->draw_layout
+					(
+					$gc,
+					($element->{X} + $mask_and_element_strip->{X_OFFSET}) * $character_width,
+					($element->{Y} + $mask_and_element_strip->{Y_OFFSET}) * $character_height,
+					$layout
+					);
+		}
+	}
+
+# draw ruler lines
+for my $line (@{$self->{RULER_LINES}})
+	{
+	my $color = Gtk2::Gdk::Color->new( map {$_ * 257} @{$line->{COLOR} }) ;
+	$self->{widget}->get_colormap->alloc_color($color,TRUE,TRUE) ;
+	
+	$gc->set_foreground($color);
+	
+	if($line->{TYPE} eq 'VERTICAL')
+		{
+		$self->{PIXMAP}->draw_line
+					(
+					$gc,
+					$line->{POSITION} * $character_width, 0,
+					$line->{POSITION} * $character_width, $widget_height
+					);
+		}
+	else
+		{
+		$self->{PIXMAP}->draw_line
+					(
+					$gc,
+					0, $line->{POSITION} * $character_height,
+					$widget_width, $line->{POSITION} * $character_height
+					);
+		}
+	}
+
+# draw connections
+my (%connected_connections, %connected_connectors) ;
+
+for my $connection (@{$self->{CONNECTIONS}})
+	{
+	my $draw_connection ;
+	my $connector  ;
+	
+	if($self->is_over_element($connection->{CONNECTED}, $self->{MOUSE_X}, $self->{MOUSE_Y}, 1))
+		{
+		$draw_connection++ ;
+		
+		$connector = $connection->{CONNECTED}->get_named_connection($connection->{CONNECTOR}{NAME}) ;
+		$connected_connectors{$connection->{CONNECTED}}{$connector->{X}}{$connector->{Y}}++ ;
+		}
+		
+	if($self->is_over_element($connection->{CONNECTEE}, $self->{MOUSE_X}, $self->{MOUSE_Y}, 1))
+		{
+		$draw_connection++ ;
+		
+		my $connectee_connection = $connection->{CONNECTEE}->get_named_connection($connection->{CONNECTION}{NAME}) ;
+		
+		if($connectee_connection)
+			{
+			$connected_connectors{$connection->{CONNECTEE}}{$connectee_connection->{X}}{$connectee_connection->{Y}}++ ;
+			}
+		}
+		
+	if($draw_connection)
+		{
+		$gc->set_foreground($self->get_color('connection'));	
+		
+		$connector ||= $connection->{CONNECTED}->get_named_connection($connection->{CONNECTOR}{NAME}) ;
+		
+		$self->{PIXMAP}->draw_rectangle
+						(
+						$gc,
+						FALSE,
+						($connector->{X} + $connection->{CONNECTED}{X}) * $character_width,
+						($connector->{Y}  + $connection->{CONNECTED}{Y}) * $character_height,
+						$character_width, $character_height
+						);
+		}
+	}
+	
+# draw connectors and connection points
+for my $element (grep {$self->is_over_element($_, $self->{MOUSE_X}, $self->{MOUSE_Y}, 1)} @{$self->{ELEMENTS}})
+	{
+	$gc->set_foreground($self->get_color('connector_point'));
+	for my $connector ($element->get_connector_points())
+		{
+		next if exists $connected_connectors{$element}{$connector->{X}}{$connector->{Y}} ;
+		
+		$self->{PIXMAP}->draw_rectangle
+						(
+						$gc,
+						FALSE,
+						($element->{X} + $connector->{X}) * $character_width,
+						($connector->{Y} + $element->{Y}) * $character_height,
+						$character_width, $character_height
+						);
+		}
+		
+	$gc->set_foreground($self->get_color('connection_point'));
+	for my $connection_point ($element->get_connection_points())
+		{
+		next if exists $connected_connections{$element}{$connection_point->{X}}{$connection_point->{Y}} ;
+		
+		$self->{PIXMAP}->draw_rectangle # little box
+						(
+						$gc,
+						TRUE,
+						(($connection_point->{X} + $element->{X}) * $character_width) + ($character_width / 3),
+						(($connection_point->{Y} + $element->{Y}) * $character_height) + ($character_height / 3),
+						$character_width / 3 , $character_height / 3
+						);
+		}
+		
+	for my $extra_point ($element->get_extra_points())
+		{
+		if(exists $extra_point ->{COLOR})
+			{
+			$gc->set_foreground($self->get_color($extra_point ->{COLOR}));
+			}
+		else
+			{
+			$gc->set_foreground($self->get_color('extra_point'));
+			}
+			
+		$self->{PIXMAP}->draw_rectangle
+						(
+						$gc,
+						FALSE,
+						(($extra_point ->{X}  + $element->{X}) * $character_width),
+						(($extra_point ->{Y}  + $element->{Y}) * $character_height),
+						$character_width, $character_height 
+						);
+		}
+	}
+
+# draw new connections
+for my $new_connection (@{$self->{NEW_CONNECTIONS}})
+	{
+	$gc->set_foreground($self->get_color('red'));
+	
+	my $end_connection = $new_connection->{CONNECTED}->get_named_connection($new_connection->{CONNECTOR}{NAME}) ;
+	
+	$self->{PIXMAP}->draw_rectangle
+					(
+					$gc,
+					FALSE,
+					($end_connection->{X} + $new_connection->{CONNECTED}{X}) * $character_width ,
+					($end_connection->{Y} + $new_connection->{CONNECTED}{Y}) * $character_height ,
+					$character_width, $character_height
+					);
+	}
+
+delete $self->{NEW_CONNECTIONS} ;
+	
+# draw selection rectangle
+if(defined $self->{SELECTION_RECTANGLE}{END_X})
+	{
+	my $start_x = $self->{SELECTION_RECTANGLE}{START_X} * $character_width ;
+	my $start_y = $self->{SELECTION_RECTANGLE}{START_Y} * $character_height ;
+	my $width = ($self->{SELECTION_RECTANGLE}{END_X} - $self->{SELECTION_RECTANGLE}{START_X}) * $character_width ;
+	my $height = ($self->{SELECTION_RECTANGLE}{END_Y} - $self->{SELECTION_RECTANGLE}{START_Y}) * $character_height; 
+	
+	if($width < 0)
+		{
+		$width *= -1 ;
+		$start_x -= $width ;
+		}
+		
+	if($height < 0)
+		{
+		$height *= -1 ;
+		$start_y -= $height ;
+		}
+		
+	$gc->set_foreground($self->get_color('selection_rectangle')) ;
+	$self->{PIXMAP}->draw_rectangle($gc, FALSE,$start_x, $start_y, $width, $height);
+	
+	delete $self->{SELECTION_RECTANGLE}{END_X} ;
+	}
+
+$widget->window->draw_drawable
+		(
+		$widget->style->fg_gc($widget->state),
+		$self->{PIXMAP},
+		$event->area->x, $event->area->y,
+		$event->area->x, $event->area->y,
+		$event->area->width, $event->area->height
+		);
+
+return TRUE;
+}
 
 #-----------------------------------------------------------------------------
 
 sub button_release_event 
 {
-my ($self, $event) = @_ ;
+my ($widget, $event, $self) = @_ ;
 
-my $modifiers = $event->{MODIFIERS} ;
+my $modifiers = get_key_modifiers($event) ;
 
 if($self->exists_action("${modifiers}-button_release"))
 	{
 	$self->run_actions(["${modifiers}-button_release", $event]) ;
-	return 1 ;
+	return TRUE ;
 	}
 
 if(defined $self->{MODIFIED_INDEX} && defined $self->{MODIFIED} && $self->{MODIFIED_INDEX} == $self->{MODIFIED})
@@ -536,11 +784,10 @@ $self->update_display();
 }
 
 #-----------------------------------------------------------------------------
-
 sub button_press_event 
 {
 #~ print "button_press_event\n" ;
-my ($self, $event, $wrapper_event) = @_ ;
+my ($widget, $event, $self) = @_ ;
 
 $self->{DRAGGING} = '' ;
 delete $self->{RESIZE_CONNECTOR_NAME} ;
@@ -548,19 +795,19 @@ delete $self->{RESIZE_CONNECTOR_NAME} ;
 $self->create_undo_snapshot() ;
 $self->{MODIFIED_INDEX} = $self->{MODIFIED} ;
 
-my $modifiers = $event->{MODIFIERS} ;
-my $button = $event->{BUTTON} ;
+my $modifiers = get_key_modifiers($event) ;
+my $button = ${event}->button() ;
 
 if($self->exists_action("${modifiers}-button_press-$button"))
 	{
 	$self->run_actions(["${modifiers}-button_press-$button", $event]) ;
-	return 1 ;
+	return TRUE ;
 	}
 
-my($x, $y) = @{$event->{COORDINATES}} ;
-
-if($event->{TYPE} eq '2button-press')
+if($event->type eq '2button-press')
 	{
+	my($x, $y) = $self->closest_character($event->coords()) ;
+	
 	my @element_over = grep { $self->is_over_element($_, $x, $y) } reverse @{$self->{ELEMENTS}} ;
 	
 	if(@element_over)
@@ -570,13 +817,14 @@ if($event->{TYPE} eq '2button-press')
 		$self->update_display();
 		}
 		
-	return 1 ;
+	return TRUE ;
 	}
 
-if($event->{BUTTON} == 1) 
+if($event->button == 1) 
 	{
-	my $modifiers = $event->{MODIFIERS} ;
+	my $modifiers = get_key_modifiers($event) ;
 	
+	my ($x, $y) = $self->closest_character($event->coords()) ;
 	my ($first_element) = first_value {$self->is_over_element($_, $x, $y)} reverse @{$self->{ELEMENTS}} ;
 	
 	if ($modifiers eq 'C00')
@@ -598,7 +846,6 @@ if($event->{BUTTON} == 1)
 				{
 				unless($self->is_element_selected($first_element))
 					{
-					# make the element under cursor the only selected element
 					$self->select_elements(0, @{$self->{ELEMENTS}}) ;
 					$self->select_elements(1, $first_element) ;
 					}
@@ -606,8 +853,7 @@ if($event->{BUTTON} == 1)
 			}
 		else
 			{
-			# deselect all
-			$self->deselect_all_elements()  if ($modifiers eq '000')  ;
+			$self->select_elements(0, @{$self->{ELEMENTS}})  if ($modifiers eq '000')  ;
 			}
 		}
 	
@@ -616,35 +862,35 @@ if($event->{BUTTON} == 1)
 	$self->update_display();
 	}
 	
-if($event->{BUTTON} == 2) 
+if($event->button == 2) 
 	{
+	my ($x, $y) = $self->closest_character($event->coords()) ;
 	$self->{SELECTION_RECTANGLE} = {START_X => $x , START_Y => $y} ;
 	
 	$self->update_display();
 	}
   
-if($event->{BUTTON} == 3) 
+if($event->button == 3) 
 	{
-	$self->display_popup_menu($wrapper_event) ; # display_popup_menu is handled by derived Asciio
+	$self->display_popup_menu($event) ;
 	}
   
-return 1;
+return TRUE;
 }
 
 #-----------------------------------------------------------------------------
 
 sub motion_notify_event 
 {
-my ($self, $event) = @_ ;
+my ($widget, $event, $self) = @_ ;
 
-my($x, $y) = @{$event->{COORDINATES}} ;
-my $modifiers = $event->{MODIFIERS} ; 
-my $button = $event->{BUTTON} ;
+my ($x, $y) = $self->closest_character($event->coords()) ;
+my $modifiers = get_key_modifiers($event) ;
 
 if($self->exists_action("${modifiers}motion_notify"))
 	{
 	$self->run_actions(["${modifiers}-motion_notify", $event]) ;
-	return 1 ;
+	return TRUE ;
 	}
 
 if($self->{PREVIOUS_X} != $x || $self->{PREVIOUS_Y} != $y)
@@ -653,7 +899,7 @@ if($self->{PREVIOUS_X} != $x || $self->{PREVIOUS_Y} != $y)
 	$self->update_display() ;
 	}
 	
-if ($event->{STATE} >= "button1-mask") 
+if ($event->state >= "button1-mask") 
 	{
 	if($self->{DRAGGING} ne '')
 		{
@@ -699,12 +945,12 @@ if ($event->{STATE} >= "button1-mask")
 		}
 	}
 
-if ($event->{STATE} >= "button2-mask") 
+if ($event->state >= "button2-mask") 
 	{
-	$self->select_element_event($x, $y, $self->{MIDDLE_BUTTON_SELECTION_FILTER} || sub{1}) ;
+	$self->select_element_event($x, $y, sub{ref $_[0] ne 'App::Asciio::stripes::section_wirl_arrow'}) ;
 	}
 	
-return 1;
+return TRUE;
 }
 
 #-----------------------------------------------------------------------------
@@ -793,71 +1039,22 @@ if($x_offset != 0 || $y_offset != 0)
 
 sub key_press_event
 {
-my ($self, $event)= @_;
+my ($widget, $event, $self)= @_;
 
-my $modifiers = $event->{MODIFIERS} ;
-my $key = $self->{KEYS}{C}{$event->{KEY_VALUE}} ;
+#~ print DumpTree \@_, '',  DISPLAY_PERL_ADDRESS => 1 ;
+#~ print "key_press_event: keyval is <" . $event->keyval() . ">\n" ;
+
+my $key = $C{$event->keyval()} ;
+my $modifiers = get_key_modifiers($event) ;
 
 $self->run_actions("$modifiers-$key") ;
 
-return 0 ;
+return FALSE;
 }
 
-#-----------------------------------------------------------------------------
-
-sub get_character_size
-{
-my ($self) = @_ ;
-	
-if(exists $self->{USER_CHARACTER_WIDTH})
-	{
-	return ($self->{USER_CHARACTER_WIDTH}, $self->{USER_CHARACTER_HEIGHT}) ;
-	}
-else
-	{
-	return (8, 16) ;
-	}
-}
-
-#-----------------------------------------------------------------------------
-
-sub set_character_size
-{
-my ($self, $width, $height) = @_ ;
-	
-($self->{USER_CHARACTER_WIDTH}, $self->{USER_CHARACTER_HEIGHT}) = ($width, $height) ;
-}
-
-#-----------------------------------------------------------------------------
-
-sub get_color 
-{
-my ($self, $name) = @_;
-
-unless (exists $self->{ALLOCATED_COLORS}{$name}) 
-	{
-	my $color  = [255, 0, 0];
-	
-	$self->{ALLOCATED_COLORS}{$name} = $color ;
-	}
-	
-return($self->{ALLOCATED_COLORS}{$name}) ;
-}
-
-#-----------------------------------------------------------------------------
-
-sub flush_color_cache
-{
-my ($self) = @_ ;
-delete $self->{ALLOCATED_COLORS} ;
-}
-
-#-----------------------------------------------------------------------------
-
-
 =head1 DEPENDENCIES
 
-gnome libraries, gtk, gtk-perl for the gtk version
+gnome libraries, gtk, gtk-perl, perl
 
 =head1 BUGS AND LIMITATIONS
 
@@ -878,7 +1075,7 @@ it and/or modify it under the same terms as Perl itself.
 
 =head2 Gentoo
 
-I run gentoo, packages to install gtk-perl exist. Install Asciio with cpan.
+I run gentoo, packages to install gtk-perl exist. Install Ascii with cpan.
 
 =head2 FreeBSD
 
@@ -899,7 +1096,7 @@ Ports are on the way.
 
 =head2 Windows
 
-B<Asciio> is part of the B<camelbox> distribution and can be found here: L<http://code.google.com/p/camelbox/>. Install, run AsciiO from the 'bin' directory.
+AsciiO is part of B<camelbox> and can be found here: L<http://code.google.com/p/camelbox/>. Install, run AsciiO from the 'bin' directory.
 
       .-------------------------------.
      /                               /|
